Enum ConnectionKind

pub enum ConnectionKind {
    Feeler,
    Regular(ServiceFlags),
    Extra,
    Manual,
}
Variants



Feeler

A feeler connection is a short-lived connection used to check whether this peer is alive.

After handshake, we ask for addresses and when we receive an answer we just disconnect, marking this peer as alive in our address manager.



Regular(ServiceFlags)

A regular peer, used to send requests to and learn about transactions and blocks.



Extra

An extra peer specially created if our tip hasn’t moved for too long.

If more than NodeContext::ASSUME_STALE seconds have passed since the last processed block, we use this to make sure we are not in a partitioned subnet, unable to learn about new blocks.



Manual

A connection that was manually requested by our user. This type of peer won’t be banned on misbehaving, and won’t respect the [ServiceFlags] requirements when creating a connection.