pub(crate) enum InflightRequests {
Headers,
UtreexoState(u32),
Blocks(BlockHash),
Connect(u32),
GetFilters,
UtreexoProof(BlockHash),
GetAddresses,
}Variants
Headers
Requests the peer to send us the next block headers in their main chain
UtreexoState(u32)
Requests the peer to send us the utreexo state for a given peer
Blocks(BlockHash)
Requests the peer to send us the block data for a given block hash
Connect(u32)
We’ve opened a connection with a peer, and are waiting for them to complete the handshake.
GetFilters
Requests the peer to send us the compact filters for blocks
UtreexoProof(BlockHash)
Requests the peer to send us the utreexo proof for a given block
GetAddresses
We’ve requested addresses from a peer