macro_rules! try_and_warn {
($what:expr) => { ... };
}Expand description
Run a task and warn any errors that might occur.
try_and_log! variant for tasks that can fail safely.
macro_rules! try_and_warn {
($what:expr) => { ... };
}Run a task and warn any errors that might occur.
try_and_log! variant for tasks that can fail safely.