Struct LocalPeerView

pub struct LocalPeerView {
Show 13 fields pub(crate) message_times: Ema, pub(crate) state: PeerStatus, pub(crate) address_id: u32, pub(crate) channel: UnboundedSender<NodeRequest>, pub(crate) services: ServiceFlags, pub(crate) user_agent: String, pub(crate) address: IpAddr, pub(crate) port: u16, pub(crate) _last_message: Instant, pub(crate) kind: ConnectionKind, pub(crate) height: u32, pub(crate) banscore: u32, pub(crate) transport_protocol: TransportProtocol,
}
Local information kept about each peer

Fields

message_times: Ema

Average message times from this peer

This is measured in milliseconds, and it’s recorded every time we get a response from a peer

state: PeerStatus

The state in which this peer is, e.g., awaiting handshake, ready, banned, etc.

address_id: u32

An id identifying this peer’s address in our address manager

channel: UnboundedSender<NodeRequest>

A channel used to send requests to this peer

services: ServiceFlags

Services this peer claims to support

user_agent: String

A version string that identifies which software this peer is running

address: IpAddr

This peer’s IP address

port: u16

The port we used to connect to this peer

_last_message: Instant

The last time we received a message from this peer

kind: ConnectionKind

The kind of connection we have with this peer

We use different connections with different goals, e.g., feeler connections, regular connections, extra connections to find about new tips, etc.

height: u32

The latest height this peer has announced to us

banscore: u32

The banscore of this peer

This is a score kept for each peer, every time this peer misbehaves, we increase this score. If the score reaches a certain threshold, we ban the peer.

transport_protocol: TransportProtocol

The transport protocol this peer is using (v1 or v2)

Implementations

Source

impl LocalPeerView

pub(crate) const fn is_manual_peer(&self) -> bool

Whether this is a manually added peer

pub(crate) const fn is_regular_peer(&self) -> bool

Whether this is a regular peer

pub(crate) const fn is_long_lived(&self) -> bool